/** @file
 *
 * @brief Plik nagwkowy.
 *
 * Pakiet @b vso-gui jest, jak nazwa wskazuje, bardzo prostym intefejsem
 * graficznym przeznaczonym dla OpenGL.  Uytkownik moe definiowa
 * przyciski aktywujace pewne akcje, przeczniki, przeczniki radiowe,
 * paski postpu, statyczne napisy oraz ramki grupujce wicej
 * elementw GUI.
 *
 * @b vso-gui jest napisany obiektowo w jzyku C (dokadniej C99).
 */

#ifndef __GUI_H_INCLUDED
#define __GUI_H_INCLUDED

/**
 * Inicjalizacja GUI.  Domylnie GUI nie jest wywietlane, naley je
 * aktywowa funkcj ShowGUI().
 *
 * @param width		pocztkowa szeroko okna
 * @param height	pocztkowa wysoko okna
 * @param ...		lista wskanikw na widgedy (pWidget), zakoczona
 *                      wskanikiem pustym
 */
void InitGUI(GLuint width, GLuint height, ...);

/**
 * Poinformowanie systemu GUI o zmianie rozmairu okna.
 *
 * @param width		nowa szeroko okna
 * @param height	nowa wysoko okna
 */
void ReshapeGUI(GLuint width, GLuint height);

/**
 * Ukrywa GUI.
 */
void HideGUI();

/**
 * Wywietla GUI.
 */
void ShowGUI();

/**
 * Rysuje GUI, o ile nie zostao wczeniej ukryte funkcj HideGUI().
 */
void DrawGUI();

/**
 * Przesya do systemu GUI informacj o klikniciu myszk w punkcie ekranu
 * o wsprzdnych (x,y).  Jeli znajduje si tam widget, zostanie
 * aktywowany (,,kliknity'') - wykonana zostanie waciwa danemu typowi
 * widgetu akcja.
 */
void ClickGUI(GLint x, GLint y);

/**
 * Dealokuje wszystkie struktury 
 */
void DestroyGUI();

/**
 * Typy widgetw.
 */
typedef enum {
	BUTTON,		/**< Przycisk */
	CHECKBOX,	/**< Przecznik */
	RADIOBUTTONS,	/**< Przecznik radiowy */
	ISLIDER,	/**< Pasek postpu (dla liczb cakowitych) */
	FSLIDER,	/**< Pasek postpu (dla liczb zmiennoprzecinkowych) */
	CONTAINER,	/**< Panel zawierajcy inne widgety */
	LABEL,		/**< Statyczny tekst */
	NONE
} WidgetType;

typedef enum {SLIDER_VERTICAL, SLIDER_HORIZONTAL} SliderOrientation;
typedef enum {RADIOBUTTONS_HORIZONTAL, RADIOBUTTONS_VERTICAL} RadioButtonsOrientation;

typedef enum {
	ALIGN_LEFT,
	ALIGN_RIGHT,
	ALIGN_TOP,
	ALIGN_BOTTOM,
	ALIGN_CENTER
} TextAlignment;

typedef struct uiWidget* pWidget;

typedef void (*activate_widget_fn)(pWidget w, GLint x, GLint y);
typedef void (*draw_widget_fn)(pWidget w, GLint dx, GLint dy);
typedef struct uiWidget {
	GLubyte		type;		/**< typ widgetu: #WidgetType */
	GLboolean	visible;	/**< widget widoczny (wywietlany) */
	GLboolean	active;		/**< widget reaguje na kliknicia */
	
	/**
	 * Numer widgetu. Moe by wykorzystany w funkcji podpitej pod
	 * kilka widgetw do ich rozrznienia.
	 */
	GLuint		id;
	
	GLint		x;		/**< wsprzdna x */
	GLint		y;		/**< wsprzdna y */
	
	GLuint		w;		/**< szeroko */
	GLuint		h;		/**< wysoko */
	
	void		*extra;		/**< dodatkowe dane specyficzne dla
					     rnych typw  widgetw */
	
	activate_widget_fn	mbp;	/**< funkcja woana przy klikniciu */
	draw_widget_fn		draw;	/**< funkcja rysujca widget */
} uiWidget;

#define ANYID 0x00

/** @brief Utworzenie nowego, pustego widgetu.
 *
 * Funkcja wstpnie inicjuje struktur uiWidget.
 * 
 * @return wskanik na uiWidget
 */
pWidget uiNewWidget(GLuint id, GLint x, GLint y, GLuint w, GLuint h);

typedef void (*button_callback)(pWidget widget);
pWidget uiNewButton(pWidget widget, button_callback callback, const char* label, GLenum halign, GLenum valign);

typedef void (*checkbox_callback)(pWidget widget, GLboolean state);
pWidget uiNewCheckBox(pWidget widget, checkbox_callback callback, const char* label, GLenum halign, GLenum valign);

pWidget uiNewContainer(pWidget widget, ...);

typedef void (*fslider_callback)(pWidget widget, GLfloat x);
pWidget uiNewFSlider(pWidget widget, fslider_callback callback, GLfloat x, GLfloat min, GLfloat max, GLenum orientation, GLfloat r, GLfloat g, GLfloat b);

typedef void (*islider_callback)(pWidget widget, GLint x);
pWidget uiNewISlider(pWidget widget, islider_callback callback, GLint x, GLint min, GLint max, GLenum orientation, GLfloat r, GLfloat g, GLfloat b);

pWidget uiNewLabel(pWidget widget, GLenum halign, GLenum interlinesep, GLfloat r, GLfloat g, GLfloat b, ...);

typedef void (*radiobuttons_callback)(pWidget widget, GLuint option);
pWidget uiNewRadioButtons(pWidget widget, radiobuttons_callback callback, GLenum halign, GLenum valign, GLenum orientation, ...);

/** Ukrycie widgetu. */
void uiHide(pWidget widget);

/** Wcznie widgetu. */
void uiShow(pWidget widget);

/** Wycznie reakcji na kliknicia danego widgetu */
void uiDeactive(pWidget widget );

/** Wcznie reakcji na kliknicia danego widgetu */
void uiActive(pWidget widget);

/** Centrowanie widgetu wzgldem okna. */
void uiCenterWidget(pWidget widget);

/* eof */

#endif
